@@ -18,15 +18,18 @@
 package org.apache.jasper;
 
 import java.io.File;
-import java.util.*;
+import java.util.Enumeration;
+import java.util.Map;
+import java.util.Properties;
 
 import javax.servlet.ServletConfig;
 import javax.servlet.ServletContext;
+import javax.servlet.jsp.tagext.TagLibraryInfo;
 
-import org.apache.jasper.compiler.TldLocationsCache;
 import org.apache.jasper.compiler.JspConfig;
-import org.apache.jasper.compiler.TagPluginManager;
 import org.apache.jasper.compiler.Localizer;
+import org.apache.jasper.compiler.TagPluginManager;
+import org.apache.jasper.compiler.TldCache;
 import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
 
@@ -40,7 +43,7 @@ import org.apache.juli.logging.LogFactory;
 public final class EmbeddedServletOptions implements Options {
 
     // Logger
-    private Log log = LogFactory.getLog(EmbeddedServletOptions.class);
+    private final Log log = LogFactory.getLog(EmbeddedServletOptions.class); // must not be static
 
     private Properties settings = new Properties();
 
@@ -60,7 +63,7 @@ public final class EmbeddedServletOptions implements Options {
     private boolean keepGenerated = true;
 
     /**
-     * Should white spaces between directives or actions be trimmed?
+     * Should template text that consists entirely of whitespace be removed?
      */
     private boolean trimSpaces = false;
 
@@ -87,7 +90,7 @@ public final class EmbeddedServletOptions implements Options {
     private int checkInterval = 0;
 
     /**
-     * Is the generation of SMAP info for JSR45 debuggin suppressed?
+     * Is the generation of SMAP info for JSR45 debugging suppressed?
      */
     private boolean isSmapSuppressed = false;
 
@@ -129,12 +132,12 @@ public final class EmbeddedServletOptions implements Options {
     /**
      * Compiler target VM.
      */
-    private String compilerTargetVM = "1.5";
+    private String compilerTargetVM = "1.7";
 
     /**
      * The compiler source VM.
      */
-    private String compilerSourceVM = "1.5";
+    private String compilerSourceVM = "1.7";
 
     /**
      * The compiler class name.
@@ -142,9 +145,9 @@ public final class EmbeddedServletOptions implements Options {
     private String compilerClassName = null;
 
     /**
-     * Cache for the TLD locations
+     * Cache for the TLD URIs, resource paths and parsed files.
      */
-    private TldLocationsCache tldLocationsCache = null;
+    private TldCache tldCache = null;
 
     /**
      * Jsp config information
@@ -184,6 +187,38 @@ public final class EmbeddedServletOptions implements Options {
     private boolean displaySourceFragment = true;
 
 
+    /**
+     * The maximum number of loaded jsps per web-application. If there are more
+     * jsps loaded, they will be unloaded.
+     */
+    private int maxLoadedJsps = -1;
+
+    /**
+     * The idle time in seconds after which a JSP is unloaded.
+     * If unset or less or equal than 0, no jsps are unloaded.
+     */
+    private int jspIdleTimeout = -1;
+
+    /**
+     * System property that controls if the strict quoting rules are applied
+     * when parsing attribute values that use scriptlet expressions (<%=...%>).
+     */
+    private static final boolean STRICT_QUOTE_ESCAPING_DEFAULT= Boolean.parseBoolean(
+            System.getProperty(
+                    "org.apache.jasper.compiler.Parser.STRICT_QUOTE_ESCAPING",
+                    "true"));
+    /**
+     * Should JSP.1.6 be applied strictly to attributes defined using scriptlet
+     * expressions?
+     */
+    private boolean strictQuoteEscaping = STRICT_QUOTE_ESCAPING_DEFAULT;
+
+    /**
+     * When EL is used in JSP attribute values, should the rules for quoting of
+     * attributes described in JSP.1.6 be applied to the expression?
+     */
+    private boolean quoteAttributeEL = true;
+
     public String getProperty(String name ) {
         return settings.getProperty( name );
     }
@@ -194,20 +229,32 @@ public final class EmbeddedServletOptions implements Options {
         }
     }
 
+    public void setQuoteAttributeEL(boolean b) {
+        this.quoteAttributeEL = b;
+    }
+
+    @Override
+    public boolean getQuoteAttributeEL() {
+        return quoteAttributeEL;
+    }
+
     /**
      * Are we keeping generated code around?
      */
+    @Override
     public boolean getKeepGenerated() {
         return keepGenerated;
     }
 
     /**
-     * Should white spaces between directives or actions be trimmed?
+     * Should template text that consists entirely of whitespace be removed?
      */
+    @Override
     public boolean getTrimSpaces() {
         return trimSpaces;
     }
 
+    @Override
     public boolean isPoolingEnabled() {
         return isPoolingEnabled;
     }
@@ -215,29 +262,23 @@ public final class EmbeddedServletOptions implements Options {
     /**
      * Are we supporting HTML mapped servlets?
      */
+    @Override
     public boolean getMappedFile() {
         return mappedFile;
     }
 
-    /**
-     * Should errors be sent to client or thrown into stderr?
-     * @deprecated
-     */
-    @Deprecated
-    public boolean getSendErrorToClient() {
-        return true;
-    }
-
     /**
      * Should class files be compiled with debug information?
      */
+    @Override
     public boolean getClassDebugInfo() {
         return classDebugInfo;
     }
 
     /**
-     * Background JSP compile thread check intervall
+     * Background JSP compile thread check interval
      */
+    @Override
     public int getCheckInterval() {
         return checkInterval;
     }
@@ -245,6 +286,7 @@ public final class EmbeddedServletOptions implements Options {
     /**
      * Modification test interval.
      */
+    @Override
     public int getModificationTestInterval() {
         return modificationTestInterval;
     }
@@ -252,6 +294,7 @@ public final class EmbeddedServletOptions implements Options {
     /**
      * Re-compile on failure.
      */
+    @Override
     public boolean getRecompileOnFail() {
         return recompileOnFail;
     }
@@ -259,13 +302,15 @@ public final class EmbeddedServletOptions implements Options {
     /**
      * Is Jasper being used in development mode?
      */
+    @Override
     public boolean getDevelopment() {
         return development;
     }
 
     /**
-     * Is the generation of SMAP info for JSR45 debuggin suppressed?
+     * Is the generation of SMAP info for JSR45 debugging suppressed?
      */
+    @Override
     public boolean isSmapSuppressed() {
         return isSmapSuppressed;
     }
@@ -273,6 +318,7 @@ public final class EmbeddedServletOptions implements Options {
     /**
      * Should SMAP info for JSR45 debugging be dumped to a file?
      */
+    @Override
     public boolean isSmapDumped() {
         return isSmapDumped;
     }
@@ -280,6 +326,7 @@ public final class EmbeddedServletOptions implements Options {
     /**
      * Are Text strings to be generated as char arrays?
      */
+    @Override
     public boolean genStringAsCharArray() {
         return this.genStringAsCharArray;
     }
@@ -287,6 +334,7 @@ public final class EmbeddedServletOptions implements Options {
     /**
      * Class ID for use in the plugin tag when the browser is IE.
      */
+    @Override
     public String getIeClassId() {
         return ieClassId;
     }
@@ -294,6 +342,7 @@ public final class EmbeddedServletOptions implements Options {
     /**
      * What is my scratch dir?
      */
+    @Override
     public File getScratchDir() {
         return scratchDir;
     }
@@ -302,6 +351,7 @@ public final class EmbeddedServletOptions implements Options {
      * What classpath should I use while compiling the servlets
      * generated from JSP files?
      */
+    @Override
     public String getClassPath() {
         return classpath;
     }
@@ -309,6 +359,7 @@ public final class EmbeddedServletOptions implements Options {
     /**
      * Is generation of X-Powered-By response header enabled/disabled?
      */
+    @Override
     public boolean isXpoweredBy() {
         return xpoweredBy;
     }
@@ -316,6 +367,7 @@ public final class EmbeddedServletOptions implements Options {
     /**
      * Compiler to use.
      */
+    @Override
     public String getCompiler() {
         return compiler;
     }
@@ -323,6 +375,7 @@ public final class EmbeddedServletOptions implements Options {
     /**
      * @see Options#getCompilerTargetVM
      */
+    @Override
     public String getCompilerTargetVM() {
         return compilerTargetVM;
     }
@@ -330,6 +383,7 @@ public final class EmbeddedServletOptions implements Options {
     /**
      * @see Options#getCompilerSourceVM
      */
+    @Override
     public String getCompilerSourceVM() {
         return compilerSourceVM;
     }
@@ -337,10 +391,12 @@ public final class EmbeddedServletOptions implements Options {
     /**
      * Java compiler class to use.
      */
+    @Override
     public String getCompilerClassName() {
         return compilerClassName;
     }
 
+    @Override
     public boolean getErrorOnUseBeanInvalidClassAttribute() {
         return errorOnUseBeanInvalidClassAttribute;
     }
@@ -349,35 +405,42 @@ public final class EmbeddedServletOptions implements Options {
         errorOnUseBeanInvalidClassAttribute = b;
     }
 
-    public TldLocationsCache getTldLocationsCache() {
-        return tldLocationsCache;
+    @Override
+    public TldCache getTldCache() {
+        return tldCache;
     }
 
-    public void setTldLocationsCache( TldLocationsCache tldC ) {
-        tldLocationsCache = tldC;
+    public void setTldCache(TldCache tldCache) {
+        this.tldCache = tldCache;
     }
 
+    @Override
     public String getJavaEncoding() {
         return javaEncoding;
     }
 
+    @Override
     public boolean getFork() {
         return fork;
     }
 
+    @Override
     public JspConfig getJspConfig() {
         return jspConfig;
     }
 
+    @Override
     public TagPluginManager getTagPluginManager() {
         return tagPluginManager;
     }
 
+    @Override
     public boolean isCaching() {
         return false;
     }
 
-    public Map getCache() {
+    @Override
+    public Map<String, TagLibraryInfo> getCache() {
         return null;
     }
 
@@ -385,10 +448,34 @@ public final class EmbeddedServletOptions implements Options {
      * Should we include a source fragment in exception messages, which could be displayed
      * to the developer ?
      */
+    @Override
     public boolean getDisplaySourceFragment() {
         return displaySourceFragment;
     }
 
+    /**
+     * Should jsps be unloaded if to many are loaded?
+     * If set to a value greater than 0 eviction of jsps is started. Default: -1
+     */
+    @Override
+    public int getMaxLoadedJsps() {
+        return maxLoadedJsps;
+    }
+
+    /**
+     * Should any jsps be unloaded when being idle for this time in seconds?
+     * If set to a value greater than 0 eviction of jsps is started. Default: -1
+     */
+    @Override
+    public int getJspIdleTimeout() {
+        return jspIdleTimeout;
+    }
+
+    @Override
+    public boolean getStrictQuoteEscaping() {
+        return strictQuoteEscaping;
+    }
+
     /**
      * Create an EmbeddedServletOptions object using data available from
      * ServletConfig and ServletContext.
@@ -396,9 +483,9 @@ public final class EmbeddedServletOptions implements Options {
     public EmbeddedServletOptions(ServletConfig config,
             ServletContext context) {
 
-        Enumeration enumeration=config.getInitParameterNames();
+        Enumeration<String> enumeration=config.getInitParameterNames();
         while( enumeration.hasMoreElements() ) {
-            String k=(String)enumeration.nextElement();
+            String k=enumeration.nextElement();
             String v=config.getInitParameter( k );
             setProperty( k, v);
         }
@@ -544,10 +631,6 @@ public final class EmbeddedServletOptions implements Options {
         }
 
         String genCharArray = config.getInitParameter("genStringAsCharArray");
-        if (genCharArray == null) {
-            // Try the old name
-            genCharArray = config.getInitParameter("genStrAsCharArray");
-        }
         if (genCharArray != null) {
             if (genCharArray.equalsIgnoreCase("true")) {
                 genStringAsCharArray = true;
@@ -594,7 +677,7 @@ public final class EmbeddedServletOptions implements Options {
             scratchDir = new File(dir);
         } else {
             // First try the Servlet 2.2 javax.servlet.context.tempdir property
-            scratchDir = (File) context.getAttribute(Constants.TMP_DIR);
+            scratchDir = (File) context.getAttribute(ServletContext.TEMPDIR);
             if (scratchDir == null) {
                 // Not running in a Servlet 2.2 container.
                 // Try to get the JDK 1.2 java.io.tmpdir property
@@ -674,9 +757,58 @@ public final class EmbeddedServletOptions implements Options {
             }
         }
 
+        String maxLoadedJsps = config.getInitParameter("maxLoadedJsps");
+        if (maxLoadedJsps != null) {
+            try {
+                this.maxLoadedJsps = Integer.parseInt(maxLoadedJsps);
+            } catch(NumberFormatException ex) {
+                if (log.isWarnEnabled()) {
+                    log.warn(Localizer.getMessage("jsp.warning.maxLoadedJsps", ""+this.maxLoadedJsps));
+                }
+            }
+        }
+
+        String jspIdleTimeout = config.getInitParameter("jspIdleTimeout");
+        if (jspIdleTimeout != null) {
+            try {
+                this.jspIdleTimeout = Integer.parseInt(jspIdleTimeout);
+            } catch(NumberFormatException ex) {
+                if (log.isWarnEnabled()) {
+                    log.warn(Localizer.getMessage("jsp.warning.jspIdleTimeout", ""+this.jspIdleTimeout));
+                }
+            }
+        }
+
+        String strictQuoteEscaping = config.getInitParameter("strictQuoteEscaping");
+        if (strictQuoteEscaping != null) {
+            if (strictQuoteEscaping.equalsIgnoreCase("true")) {
+                this.strictQuoteEscaping = true;
+            } else if (strictQuoteEscaping.equalsIgnoreCase("false")) {
+                this.strictQuoteEscaping = false;
+            } else {
+                if (log.isWarnEnabled()) {
+                    log.warn(Localizer.getMessage("jsp.warning.strictQuoteEscaping",
+                            Boolean.toString(STRICT_QUOTE_ESCAPING_DEFAULT)));
+                }
+            }
+        }
+
+        String quoteAttributeEL = config.getInitParameter("quoteAttributeEL");
+        if (quoteAttributeEL != null) {
+            if (quoteAttributeEL.equalsIgnoreCase("true")) {
+                this.quoteAttributeEL = true;
+            } else if (quoteAttributeEL.equalsIgnoreCase("false")) {
+                this.quoteAttributeEL = false;
+            } else {
+                if (log.isWarnEnabled()) {
+                    log.warn(Localizer.getMessage("jsp.warning.quoteAttributeEL"));
+                }
+            }
+        }
+
         // Setup the global Tag Libraries location cache for this
         // web-application.
-        tldLocationsCache = new TldLocationsCache(context);
+        tldCache = TldCache.getInstance(context);
 
         // Setup the jsp config info for this web app.
         jspConfig = new JspConfig(context);
